function varargout = gui(varargin)
% GUI MATLAB code for gui.fig
%      GUI, by itself, creates a new GUI or raises the existing
%      singleton*.
%
%      H = GUI returns the handle to a new GUI or the handle to
%      the existing singleton*.
%
%      GUI('CALLBACK',hObject,eventData,handles,...) calls the local
%      function named CALLBACK in GUI.M with the given input arguments.
%
%      GUI('Property','Value',...) creates a new GUI or raises the
%      existing singleton*.  Starting from the left, property value pairs are
%      applied to the GUI before gui_OpeningFcn gets called.  An
%      unrecognized property name or invalid value makes property application
%      stop.  All inputs are passed to gui_OpeningFcn via varargin.
%
%      *See GUI Options on GUIDE's Tools menu.  Choose "GUI allows only one
%      instance to run (singleton)".
%
% See also: GUIDE, GUIDATA, GUIHANDLES

%% Copyrights
% (C) All rights reserved.
%
% The code may be used free of charge for non-commercial and
% educational purposes, the only requirement is that this text is
% preserved within the derivative work. For any other purpose you
% must contact the authors for permission. This code may not be
% redistributed without written permission from the authors.
% 
% ABOUT:
% This software implements our approach to detect changes in multi-variate
% time series
% 
% IMPORTANT:
% If you use this software you should cite the following in any resulting publication: 
% [1] Michał Staniszewski, Agnieszka Skorupa, Łukasz Boguszewicz, Maria Sokół and Andrzej Polański. Quality Control Procedure Based on Partitioning of NMR Time Series.
%%

% Last Modified by GUIDE v2.5 03-Jun-2016 23:27:46

% Begin initialization code - DO NOT EDIT
warning('off','all')
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @gui_OpeningFcn, ...
                   'gui_OutputFcn',  @gui_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT


% --- Executes just before gui is made visible.
function gui_OpeningFcn(hObject, eventdata, handles, varargin)
% This function has no output args, see OutputFcn.
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% varargin   command line arguments to gui (see VARARGIN)

    % Choose default command line output for gui
    handles.output = hObject;
    handles.newline = [];
    
    %initial position in array
    handles.row = 2;
    handles.col = 2;

    % Read from the .mat file
    % Read from the .mat or .xls file
    [handles.dirname,pathname] = uigetfile({'*.mat';'*.xls'},'Select mat file or csv spreadsheet');
    if isequal(handles.dirname,0)
        disp('User selected Cancel')
        handles.varnames{1,1} = 'no data';
        handles.data{2,1} = 'no data';
        handles.data{1,2} = 'no data';
        handles.data{2,2} = ones(50,1);
    elseif(strcmp(handles.dirname((end-3):end),'.mat') == 1)
        disp(['User selected ', handles.dirname])
        handles.L = load( fullfile( pathname, handles.dirname ) );
        handles.varnames = fieldnames(handles.L);
        data_tmp = getfield(handles.L,handles.varnames{1});
        %check if data does not contain any char
        if(check_data(data_tmp))
            handles.data = data_tmp;
        else
            disp('wrong data')
            handles.data{2,1} = cell(0);
            handles.varnames{1,1} = 'no data';
            handles.data{2,1} = 'no data';
            handles.data{1,2} = 'no data';
            handles.data{2,2} = ones(50,1);
        end
    elseif (strcmp(handles.dirname((end-3):end),'.xls') == 1)
        disp(['User selected ', handles.dirname])
        M = xlsread(fullfile( pathname, handles.dirname ));
        handles.varnames{1,1} = 'single xls';
        handles.data{2,1} = 'single xls';
        for i=1:size(M,2)
            handles.data{1,i+1} = num2str(i);
            %check if data does not contain any char
            if (sum(isnan(M(:,i)))==0)
                handles.data{2,i+1} = M(:,i);
            else
                disp('wrong data')
                handles.data{2,1} = cell(0);
                handles.varnames{1,1} = 'no data';
                handles.data{2,1} = 'no data';
                handles.data{1,2} = 'no data';
                handles.data{2,2} = ones(50,1);
                break
            end
        end 
    end
    handles.cur_varnames = handles.varnames{1};
    handles.title_on_ax = strcat(handles.dirname, '->', handles.cur_varnames, '->', handles.data{2,1}, '->', handles.data{1,2});
    
    % Put all data into the table
    handles.y = put_into_table(handles,handles.data);
    set(handles.popupmenu1, 'String', handles.varnames);
    
    % Init sliders with step, minimum and maximum
    init_sliders(hObject, handles); 
    
    % Compute segments, change points, update texts and sliders
    handles = compute_all(hObject,handles,1);
    
    guidata(hObject, handles);
    

% UIWAIT makes gui wait for user response (see UIRESUME)
% uiwait(handles.figure1);


% --- Outputs from this function are returned to the command line.
function varargout = gui_OutputFcn(hObject, eventdata, handles) 
% varargout  cell array for returning output args (see VARARGOUT);
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Get default command line output from handles structure
varargout{1} = handles.output;


% --- Executes on slider movement.
function slider2_Callback(hObject, eventdata, handles)
% hObject    handle to slider2 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'Value') returns position of slider
%        get(hObject,'Min') and get(hObject,'Max') to determine range of slider
    % Number of change point slider
    handles.K_gr = get(hObject,'Value');
    % Calculate change points
    handles.pkt = calc_change_points(handles.y, handles.K_gr);
    
    handles = compute_all(hObject,handles,0);
    
    guidata(hObject, handles);

%guidata(hObject, handles);


% --- Executes during object creation, after setting all properties.
function slider2_CreateFcn(hObject, eventdata, handles)
% hObject    handle to slider2 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: slider controls usually have a light gray background.
if isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor',[.9 .9 .9]);
end


% --- Executes when selected cell(s) is changed in uitable1.
function uitable1_CellSelectionCallback(hObject, eventdata, handles)
% hObject    handle to uitable1 (see GCBO)
% eventdata  structure with the following fields (see UITABLE)
%	Indices: row and column indices of the cell(s) currently selecteds
% handles    structure with handles and user data (see GUIDATA)
    
    if (isempty(eventdata.Indices))
        handles.row = 2;
        handles.col = 2;
    else
        handles.row = eventdata.Indices(1)+1;
        handles.col = eventdata.Indices(2)+1;
    end
    handles.y = handles.data{handles.row,handles.col};
    handles.title_on_ax = strcat(handles.dirname, '->', handles.cur_varnames, '->', handles.data{handles.row,1}, '->', handles.data{1,handles.col});
    
    % Compute segments, change points, update texts and sliders
    if ~(isempty(eventdata.Indices))
        handles = compute_all(hObject,handles,1);
    end
    
    guidata(hObject, handles);

% --- Executes on button press in pushbutton2.
function pushbutton2_Callback(hObject, eventdata, handles)
% hObject    handle to pushbutton2 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
    
    handles.varnames = {};
    handles.data = {};
    % Read from the .mat file
    % Read from the .mat and .xls file
    [handles.dirname,pathname] = uigetfile({'*.mat';'*.xls'},'Select mat file or csv spreadsheet');
    if isequal(handles.dirname,0)
        disp('User selected Cancel')
        handles.varnames{1,1} = 'no data';
        handles.data{2,1} = 'no data';
        handles.data{1,2} = 'no data';
        handles.data{2,2} = ones(50,1);
    elseif(strcmp(handles.dirname((end-3):end),'.mat') == 1)
        disp(['User selected ', handles.dirname])
        handles.L = load( fullfile( pathname, handles.dirname ) );
        handles.varnames = fieldnames(handles.L);
        data_tmp = getfield(handles.L,handles.varnames{1});
        %check if data does not contain any char
        if(check_data(data_tmp))
            handles.data = data_tmp;
        else
            disp('wrong data')
            handles.data{2,1} = cell(0);
            handles.varnames{1,1} = 'no data';
            handles.data{2,1} = 'no data';
            handles.data{1,2} = 'no data';
            handles.data{2,2} = ones(50,1);
        end
    elseif (strcmp(handles.dirname((end-3):end),'.xls') == 1)
        disp(['User selected ', handles.dirname])
        M = xlsread(fullfile( pathname, handles.dirname ));
        handles.varnames{1,1} = 'single xls';
        handles.data{2,1} = 'single xls';
        for i=1:size(M,2)
            handles.data{1,i+1} = num2str(i);
            %check if data does not contain any char
            if (sum(isnan(M(:,i)))==0)
                handles.data{2,i+1} = M(:,i);
            else
                disp('wrong data')
                handles.data{2,1} = cell(0);
                handles.varnames{1,1} = 'no data';
                handles.data{2,1} = 'no data';
                handles.data{1,2} = 'no data';
                handles.data{2,2} = ones(50,1);
                break
            end
        end  
    end
    handles.cur_varnames = handles.varnames{1};
    handles.title_on_ax = strcat(handles.dirname, '->', handles.cur_varnames, '->', handles.data{2,1}, '->', handles.data{1,2});
    
    % Put all data into the table
    handles.y = put_into_table(handles,handles.data);
    set(handles.popupmenu1, 'String', handles.varnames);
    
    % Compute segments, change points, update texts and sliders
    handles = compute_all(hObject, handles,1);
    
    guidata(hObject, handles);


% --- Executes on selection change in popupmenu1.
function popupmenu1_Callback(hObject, eventdata, handles)
% hObject    handle to popupmenu1 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: contents = cellstr(get(hObject,'String')) returns popupmenu1 contents as cell array
%        contents{get(hObject,'Value')} returns selected item from popupmenu1
    % Change .data if in .mat was more celss
    if isequal(handles.dirname,0)
        
    elseif(strcmp(handles.dirname((end-3):end),'.mat') == 1)
        contents = cellstr(get(hObject,'String'));
        handles.data = getfield(handles.L,contents{get(hObject,'Value')});
        handles.cur_varnames = handles.varnames{get(hObject,'Value')};
        handles.title_on_ax = strcat(handles.dirname, '->', handles.cur_varnames, '->', handles.data{2,1}, '->', handles.data{1,2});

        % Put all data into the table
        handles.y = put_into_table(handles,handles.data);

        % Compute segments, change points, update texts and sliders
        handles = compute_all(hObject, handles,1);

        guidata(hObject, handles);
    end

% --- Executes during object creation, after setting all properties.
function popupmenu1_CreateFcn(hObject, eventdata, handles)
% hObject    handle to popupmenu1 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: popupmenu controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes on button press in pushbutton3.
function pushbutton3_Callback(hObject, eventdata, handles)
% hObject    handle to pushbutton3 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
    handles.file_name = strcat(handles.cur_varnames, '_', handles.data{handles.row,1}, '_', handles.data{1,handles.col});
    [FileName,PathName] = uiputfile({'*.png'},'Save plot', handles.file_name);
    file_Name=fullfile(PathName,FileName);
    newfig1 = figure('Visible','off'); 
    set(newfig1, 'PaperPosition', [0 0 35 15])
    copyobj(handles.axes1, newfig1);
    print(newfig1,file_Name,'-dpng')
    delete(newfig1)

function init_sliders(hObject, handles)
    
    sliderMin2 = 0;
    sliderMax2 = 50; % this is variable
    sliderStep2 = [1,1]/(sliderMax2 - sliderMin2); % major and minor steps of 1
    set(handles.slider2, 'Min', sliderMin2);
    set(handles.slider2, 'Max', sliderMax2);
    set(handles.slider2, 'SliderStep', sliderStep2);
    
    guidata(hObject, handles);

function pkt = calc_change_points(y, K_gr)
    if(K_gr>0)
        [~,pkt1] = dyn_pr_split(y,K_gr);
        pkt = pkt1(K_gr,:);
    else
        pkt=[];
    end
    
function data_checked = check_data(data)
    % Check if the data does not contain any char
    data_checked = 1;
    for i=2:size(data,1)
        for j=2:size(data,2)
            if (~isnumeric(data{i,j}))
                data_checked = 0;
                break
            end
        end
    end
                
function handles = plot_on_ax(hObject,handles)
    cla(handles.axes1)
    if handles.K_gr > 0
        [handles.conf_level,~,handles.rect]=gen_ar(handles.y,100,handles.K_gr,0,0,0);
        columns = {'ch_pt','median','down','up','from','to','correction','mean'};
        set(handles.uitable3,'data',handles.conf_level,'ColumnName',columns,'ColumnWidth',{50});
    else
        [handles.rect(1,1).x,handles.rect(1,1).y]=gen_rect(handles.y,1,size(handles.y,1));
        set(handles.uitable3,'data',[0,0,0,0,0,0,0,mean(handles.y)]);
    end
    
    cc=hsv(handles.K_gr+1);
    handles.axes1;
    hold on
    for j=1:(handles.K_gr+1)
        fill(handles.rect(j,1).x,handles.rect(j,1).y, cc(j,:),'EdgeColor',cc(j,:))
    end
    plot(handles.y,'linewidth',3,'color','k')
    title(handles.title_on_ax)
    guidata(hObject, handles);
    
function handles = plot_on_ax_chp(hObject,handles,chp)
    cla(handles.axes1)
    columns = {'ch_pt','median','down','up','from','to','correction','mean'};
    conf_level_tmp = handles.conf_level;
    conf_level_tmp(:,5:6) = handles.conf_level(:,5:6)/handles.conf_level(chp,8);
    conf_level_tmp(:,7) = round((conf_level_tmp(:,5)/conf_level_tmp(chp,5)) *100);
    if handles.K_gr > 0
        set(handles.uitable3,'data',conf_level_tmp,'ColumnName',columns,'ColumnWidth',{50});
    else
        set(handles.uitable3,'data',[0,0,0,0,0,0,0,mean(handles.y)]);
    end
    
    cc=hsv(handles.K_gr+1);
    handles.axes1;
    hold on
    for j=1:(handles.K_gr+1)
        fill(handles.rect(j,1).x,handles.rect(j,1).y/handles.conf_level(chp,8), cc(j,:),'EdgeColor',cc(j,:))
    end
    plot(handles.y/handles.conf_level(chp,8),'linewidth',3,'color','k')
    guidata(hObject, handles);

    
function handles = update_slider_text(handles)
    set(handles.text2, 'String', strcat('Nb of change points: ',num2str(handles.K_gr)));
    set(handles.slider2, 'Value', handles.K_gr); % set to beginning of sequence
    if iscell(handles.title_on_ax)
        handles.tmp{1, 1} = strcat(handles.title_on_ax{1,1} ,' ,nb of change points: ',num2str(handles.K_gr));
    else
        handles.tmp{1, 1} = strcat(handles.title_on_ax ,' ,nb of change points: ',num2str(handles.K_gr));
    end
    handles.tmp{2, 1} = num2str(handles.pkt);
    handles.newline = [handles.tmp, handles.newline];
    set(handles.edit1, 'String', handles.newline);

function y = put_into_table(handles,data)
    data_correct = 1;
    size_data = size(data);
    dane = ones(size_data-1);
    columns = cell(1,size_data(2)-1);
    rows = cell(size_data(1)-1,1);
    for i=2:1:(size_data(2))
        if (ischar(data{1,i}))
            columns{1,i-1} = data{1,i};
        else
            data_correct = 0;
            break;
        end
    end
    for i=2:1:(size_data(1))
        if (ischar(data{i,1}))
            rows{i-1,1} = data{i,1};
        else
            data_correct = 0;
            break;
        end
    end
    if(data_correct)
        set(handles.uitable1,'data',dane,'ColumnName',columns,'RowName',rows);
        y = data{2,2};
    else
        disp('Wrong data')
        handles.varnames{1,1} = 'no data';
        handles.data{2,1} = 'no data';
        handles.data{1,2} = 'no data';
        set(handles.uitable1,'data',1,'ColumnName','no data','RowName','no data');
        handles.data{2,2} = ones(50,1);
        y = ones(50,1);
    end

function handles = compute_all(hObject,handles,ch_p)
    % Calculate segments
    if(ch_p)
        [handles.K_gr] = compute_nb_cp(handles.y,0);
        % Calculate change points
        handles.pkt = calc_change_points(handles.y, handles.K_gr);
    else
        handles.chp_names = cell(0);
    end
    handles.chp_names = cell(0);
    for i=1:handles.K_gr+1
        handles.chp_names{i,1} = num2str(i);
    end
    set(handles.popupmenu3, 'Value', 1);
    set(handles.popupmenu3, 'String', handles.chp_names);
    % Plot data on axes1
    handles = plot_on_ax(hObject, handles);
    
    % Update all sliders and text box
    handles = update_slider_text(handles);

function edit1_Callback(hObject, eventdata, handles)
% hObject    handle to edit1 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of edit1 as text
%        str2double(get(hObject,'String')) returns contents of edit1 as a double


% --- Executes during object creation, after setting all properties.
function edit1_CreateFcn(hObject, eventdata, handles)
% hObject    handle to edit1 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes on selection change in popupmenu3.
function popupmenu3_Callback(hObject, eventdata, handles)
% hObject    handle to popupmenu3 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: contents = cellstr(get(hObject,'String')) returns popupmenu3 contents as cell array
%        contents{get(hObject,'Value')} returns selected item from popupmenu3

    contents = cellstr(get(hObject,'String'));
    chp = contents{get(hObject,'Value')};
    plot_on_ax_chp(hObject,handles,str2num(chp));
    

% --- Executes during object creation, after setting all properties.
function popupmenu3_CreateFcn(hObject, eventdata, handles)
% hObject    handle to popupmenu3 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: popupmenu controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes on button press in pushbutton6.
function pushbutton6_Callback(hObject, eventdata, handles)
% hObject    handle to pushbutton6 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
    plot_on_ax(hObject,handles);
